import { type ClassValue, clsx } from "clsx"
import { twMerge } from "tailwind-merge"
 
export function cn(...inputs: ClassValue[]) {
  return twMerge(clsx(inputs))
}

export function formatDate(dateString: string, format?: string): string {
  if (!dateString) return '';
  
  // Get the format from localStorage if not provided
  if (!format) {
    format = localStorage.getItem('settings_dateFormat') || 'yyyy-mm-dd';
  }
  
  try {
    // Handle different date formats
    let date;
    
    // Handle European date format (DD/MM/YYYY)
    if (dateString.includes('/')) {
      const parts = dateString.split('/');
      if (parts.length === 3) {
        // Rearrange to YYYY-MM-DD for proper parsing
        dateString = `${parts[2]}-${parts[1]}-${parts[0]}`;
      }
    }
    
    // Replace underscores with spaces
    if (dateString.includes('_')) {
      dateString = dateString.replace('_', ' ');
    }
    
    date = new Date(dateString);
    
    // Check if valid date
    if (isNaN(date.getTime())) {
      return dateString; // Return original if parsing fails
    }
    
    // Format based on user preference
    if (format === 'dd-mm-yyyy') {
      return new Intl.DateTimeFormat('en-GB', {
        year: 'numeric',
        month: '2-digit',
        day: '2-digit',
      }).format(date).replace(/\//g, '-');
    } else if (format === 'mm-dd-yyyy') {
      return new Intl.DateTimeFormat('en-US', {
        year: 'numeric',
        month: '2-digit',
        day: '2-digit',
      }).format(date).replace(/\//g, '-');
    } else {
      // Default yyyy-mm-dd format
      return date.toISOString().split('T')[0];
    }
  } catch (error) {
    console.error('Error formatting date:', error);
    return dateString;
  }
}

export function formatTime(timeString: string): string {
  if (!timeString) return '';
  
  try {
    // For times that already include colons
    if (timeString.includes(':')) {
      // Handle time format like '00:36:00'
      if (timeString.split(':').length === 3) {
        const [hours, minutes] = timeString.split(':');
        return `${hours}:${minutes}`;
      }
      
      // For time with just hours and minutes (HH:MM)
      if (timeString.split(':').length === 2) {
        return timeString;
      }
    }
    
    // Handle time without formatting or with non-standard formatting
    if (timeString.length === 4) {
      // If it's a 4-digit string like "0900", format as "09:00"
      return `${timeString.substring(0, 2)}:${timeString.substring(2, 4)}`;
    }
    
    return timeString;
  } catch (error) {
    console.error('Error formatting time:', error);
    return timeString;
  }
}

export function formatIncidentNumber(num: string): string {
  if (!num) return '';
  
  // Remove any decimal places from the incident number
  const cleanNum = num.includes('.') ? num.split('.')[0] : num;
  
  // Return the clean number without any formatting
  return cleanNum;
}

export function getInitials(name: string): string {
  if (!name) return 'U';
  
  const parts = name.split(' ');
  if (parts.length === 1) return parts[0].charAt(0).toUpperCase();
  
  return (parts[0].charAt(0) + parts[parts.length - 1].charAt(0)).toUpperCase();
}

export function truncateText(text: string, maxLength: number): string {
  if (!text) return '';
  if (text.length <= maxLength) return text;
  
  return text.slice(0, maxLength) + '...';
}

export function downloadCSV(csvContent: string, filename: string = 'incidents.csv'): void {
  const blob = new Blob([csvContent], { type: 'text/csv;charset=utf-8;' });
  const url = URL.createObjectURL(blob);
  const link = document.createElement('a');
  
  link.setAttribute('href', url);
  link.setAttribute('download', filename);
  link.style.visibility = 'hidden';
  
  document.body.appendChild(link);
  link.click();
  document.body.removeChild(link);
}

// Common areas/towns for dropdown selection
export const COMMON_AREAS = [
  { value: "Basingstoke", label: "Basingstoke" },
  { value: "Andover", label: "Andover" },
  { value: "Fleet", label: "Fleet" },
  { value: "Alton", label: "Alton" },
  { value: "Tadley", label: "Tadley" },
  { value: "Whitchurch", label: "Whitchurch" },
  { value: "Overton", label: "Overton" },
  { value: "Kingsclere", label: "Kingsclere" },
  { value: "Odiham", label: "Odiham" },
  { value: "Hartley Wintney", label: "Hartley Wintney" },
  { value: "Hook", label: "Hook" }
];
