import React, { useEffect } from "react";
import { useForm } from "react-hook-form";
import { zodResolver } from "@hookform/resolvers/zod";
import { useQuery, useMutation, useQueryClient } from "@tanstack/react-query";
import { useLocation, useParams } from "wouter";
import { Helmet } from "react-helmet";
import { apiRequest } from "@/lib/queryClient";
import { updateIncidentSchema, Incident } from "@shared/schema";
import { Button } from "@/components/ui/button";
import { Input } from "@/components/ui/input";
import { Textarea } from "@/components/ui/textarea";
import { Checkbox } from "@/components/ui/checkbox";
import { Card, CardContent } from "@/components/ui/card";
import { Form, FormControl, FormDescription, FormField, FormItem, FormLabel, FormMessage } from "@/components/ui/form";
import { Select, SelectContent, SelectItem, SelectTrigger, SelectValue } from "@/components/ui/select";
import { toast } from "@/hooks/use-toast";
import { INCIDENT_TYPE_OPTIONS, WATCH_OPTIONS } from "@/lib/incident-types";
import { COMMON_AREAS } from "@/lib/utils";
import { ArrowLeft, Save, Trash2 } from "lucide-react";
import { 
  AlertDialog,
  AlertDialogAction,
  AlertDialogCancel,
  AlertDialogContent,
  AlertDialogDescription,
  AlertDialogFooter,
  AlertDialogHeader,
  AlertDialogTitle,
  AlertDialogTrigger,
} from "@/components/ui/alert-dialog";
import { Skeleton } from "@/components/ui/skeleton";

export default function EditIncident() {
  const { id } = useParams();
  const incidentId = parseInt(id);
  const [_, navigate] = useLocation();
  const queryClient = useQueryClient();
  
  // Fetch incident data
  const { data: incident, isLoading, isError } = useQuery<Incident>({
    queryKey: [`/api/incidents/${incidentId}`],
    enabled: !isNaN(incidentId),
  });
  
  // Set up form with zod resolver
  const form = useForm({
    resolver: zodResolver(updateIncidentSchema),
    defaultValues: {
      id: incidentId,
      incidentNumber: "",
      date: "",
      time: "",
      reference: "",
      location: "",
      town: "",
      station: "",
      officerInCharge: "",
      watch: "",
      pumps: 0,
      waterCarrier: 0,
      supportPump: 0,
      aerialLadder: 0,
      rescueSupport: 0,
      incidentCommand: 0,
      incidentType: "",
      notes: "",
      safetyCheck: false,
      additionalNotes: "",
      safetyCheckComplete: "",
    },
  });
  
  // Update form values when incident data is loaded
  useEffect(() => {
    if (incident) {
      // Reset form with incident data
      form.reset({
        id: incidentId,
        ...incident,
      });
    }
  }, [incident, form, incidentId]);
  
  // Update mutation
  const updateMutation = useMutation({
    mutationFn: async (values: any) => {
      const res = await apiRequest("PUT", `/api/incidents/${incidentId}`, values);
      return res.json();
    },
    onSuccess: () => {
      queryClient.invalidateQueries({ queryKey: [`/api/incidents/${incidentId}`] });
      queryClient.invalidateQueries({ queryKey: ["/api/incidents"] });
      queryClient.invalidateQueries({ queryKey: ["/api/stats"] });
      toast({
        title: "Incident updated",
        description: "The incident has been successfully updated.",
      });
    },
    onError: (error: any) => {
      toast({
        title: "Error",
        description: error.message || "An error occurred while updating the incident.",
        variant: "destructive",
      });
    },
  });
  
  // Delete mutation
  const deleteMutation = useMutation({
    mutationFn: async () => {
      await apiRequest("DELETE", `/api/incidents/${incidentId}`);
    },
    onSuccess: () => {
      queryClient.invalidateQueries({ queryKey: ["/api/incidents"] });
      queryClient.invalidateQueries({ queryKey: ["/api/stats"] });
      toast({
        title: "Incident deleted",
        description: "The incident has been successfully deleted.",
      });
      navigate("/incidents");
    },
    onError: (error: any) => {
      toast({
        title: "Error",
        description: error.message || "An error occurred while deleting the incident.",
        variant: "destructive",
      });
    },
  });
  
  // Form submission
  const onSubmit = (values: any) => {
    console.log("Form values before submission:", values);
    // Ensure incident type is preserved in the submission
    if (!values.incidentType && incident?.incidentType) {
      values.incidentType = incident.incidentType;
    }
    console.log("Form values after adjustment:", values);
    updateMutation.mutate(values);
  };
  
  // Handle delete
  const handleDelete = () => {
    deleteMutation.mutate();
  };
  
  if (isError) {
    return (
      <div className="p-4 md:p-6">
        <Card className="max-w-xl mx-auto p-6">
          <div className="text-center space-y-4">
            <h2 className="text-2xl font-bold text-destructive">Error</h2>
            <p>The incident could not be found or an error occurred.</p>
            <Button onClick={() => navigate("/incidents")}>Back to Incidents</Button>
          </div>
        </Card>
      </div>
    );
  }
  
  return (
    <>
      <Helmet>
        <title>Edit Incident | Emergency Response Incident Management System</title>
        <meta name="description" content="Edit an existing emergency response incident record" />
      </Helmet>
      
      <div className="p-4 md:p-6">
        <div className="mb-6">
          <div className="flex flex-col md:flex-row md:items-center md:justify-between gap-4">
            <div className="flex items-center">
              <Button variant="ghost" className="mr-2" onClick={() => navigate(-1)}>
                <ArrowLeft className="h-4 w-4" />
              </Button>
              <div>
                <h1 className="text-2xl font-bold text-neutral-800">
                  {isLoading ? (
                    <Skeleton className="h-8 w-48" />
                  ) : (
                    `Edit Incident: ${incident?.incidentNumber}`
                  )}
                </h1>
                <p className="text-neutral-600">Update incident information</p>
              </div>
            </div>
            <div>
              <AlertDialog>
                <AlertDialogTrigger asChild>
                  <Button variant="destructive">
                    <Trash2 className="h-4 w-4 mr-2" />
                    Delete
                  </Button>
                </AlertDialogTrigger>
                <AlertDialogContent>
                  <AlertDialogHeader>
                    <AlertDialogTitle>Are you sure?</AlertDialogTitle>
                    <AlertDialogDescription>
                      This action cannot be undone. This will permanently delete the incident record.
                    </AlertDialogDescription>
                  </AlertDialogHeader>
                  <AlertDialogFooter>
                    <AlertDialogCancel>Cancel</AlertDialogCancel>
                    <AlertDialogAction
                      onClick={handleDelete}
                      className="bg-destructive text-destructive-foreground"
                    >
                      Delete
                    </AlertDialogAction>
                  </AlertDialogFooter>
                </AlertDialogContent>
              </AlertDialog>
            </div>
          </div>
        </div>
        
        {isLoading ? (
          <Card>
            <CardContent className="p-6">
              <div className="space-y-6">
                <div className="grid grid-cols-1 md:grid-cols-2 gap-6">
                  <Skeleton className="h-10 w-full" />
                  <Skeleton className="h-10 w-full" />
                  <Skeleton className="h-10 w-full" />
                  <Skeleton className="h-10 w-full" />
                </div>
                <Skeleton className="h-10 w-full" />
                <Skeleton className="h-32 w-full" />
              </div>
            </CardContent>
          </Card>
        ) : (
          <Card className="max-w-4xl mx-auto">
            <CardContent className="p-6">
              <Form {...form}>
                <form onSubmit={form.handleSubmit(onSubmit)} className="space-y-6">
                  <div className="grid grid-cols-1 md:grid-cols-2 gap-6">
                    {/* Incident ID */}
                    <FormField
                      control={form.control}
                      name="incidentNumber"
                      render={({ field }) => (
                        <FormItem>
                          <FormLabel>Incident ID</FormLabel>
                          <FormControl>
                            <Input placeholder="e.g. HH032473" {...field} />
                          </FormControl>
                          <FormMessage />
                        </FormItem>
                      )}
                    />
                    
                    {/* Date */}
                    <FormField
                      control={form.control}
                      name="date"
                      render={({ field }) => (
                        <FormItem>
                          <FormLabel>Date</FormLabel>
                          <FormControl>
                            <Input type="date" {...field} />
                          </FormControl>
                          <FormMessage />
                        </FormItem>
                      )}
                    />
                    
                    {/* Time */}
                    <FormField
                      control={form.control}
                      name="time"
                      render={({ field }) => (
                        <FormItem>
                          <FormLabel>Time</FormLabel>
                          <FormControl>
                            <Input type="time" {...field} />
                          </FormControl>
                          <FormMessage />
                        </FormItem>
                      )}
                    />
                    
                    {/* Incident Type */}
                    <FormField
                      control={form.control}
                      name="incidentType"
                      render={({ field }) => (
                        <FormItem>
                          <FormLabel>Incident Type</FormLabel>
                          <Select 
                            onValueChange={field.onChange} 
                            value={field.value || incident?.incidentType || ""}
                            defaultValue={incident?.incidentType || ""}
                          >
                            <FormControl>
                              <SelectTrigger>
                                <SelectValue placeholder="Select incident type" />
                              </SelectTrigger>
                            </FormControl>
                            <SelectContent>
                              {INCIDENT_TYPE_OPTIONS.map(option => (
                                <SelectItem key={option.value} value={option.value}>
                                  {option.label}
                                </SelectItem>
                              ))}
                            </SelectContent>
                          </Select>
                          <FormMessage />
                          {!field.value && incident?.incidentType && (
                            <input 
                              type="hidden" 
                              name="incidentType" 
                              value={incident.incidentType} 
                            />
                          )}
                        </FormItem>
                      )}
                    />
                    
                    {/* Location */}
                    <FormField
                      control={form.control}
                      name="location"
                      render={({ field }) => (
                        <FormItem className="md:col-span-2">
                          <FormLabel>Location</FormLabel>
                          <FormControl>
                            <Input placeholder="Address or location details" {...field} />
                          </FormControl>
                          <FormMessage />
                        </FormItem>
                      )}
                    />
                    
                    {/* Town/City */}
                    <FormField
                      control={form.control}
                      name="town"
                      render={({ field }) => {
                        // Use local state to manage custom input
                        const [isCustom, setIsCustom] = React.useState(
                          field.value && !COMMON_AREAS.some(area => area.value === field.value) && field.value !== "other"
                        );
                        const [customValue, setCustomValue] = React.useState(
                          isCustom ? field.value : ""
                        );
                        
                        return (
                          <FormItem>
                            <FormLabel>Town / Area</FormLabel>
                            {!isCustom ? (
                              <Select 
                                onValueChange={(value) => {
                                  if (value === "other") {
                                    setIsCustom(true);
                                  } else {
                                    field.onChange(value);
                                  }
                                }} 
                                value={field.value || ""}
                              >
                                <FormControl>
                                  <SelectTrigger>
                                    <SelectValue placeholder="Select area" />
                                  </SelectTrigger>
                                </FormControl>
                                <SelectContent>
                                  {COMMON_AREAS.map(option => (
                                    <SelectItem key={option.value} value={option.value}>
                                      {option.label}
                                    </SelectItem>
                                  ))}
                                  <SelectItem value="other">Other (Custom)</SelectItem>
                                </SelectContent>
                              </Select>
                            ) : (
                              <div className="flex flex-col gap-2">
                                <div className="flex gap-2">
                                  <Input
                                    placeholder="Enter custom area"
                                    value={customValue}
                                    onChange={(e) => {
                                      const newValue = e.target.value;
                                      setCustomValue(newValue);
                                      field.onChange(newValue);
                                    }}
                                    autoFocus
                                  />
                                  <Button 
                                    type="button" 
                                    variant="outline" 
                                    size="icon"
                                    onClick={() => {
                                      setIsCustom(false);
                                      setCustomValue("");
                                      field.onChange(""); // Reset to empty so placeholder shows
                                    }}
                                  >
                                    <svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" viewBox="0 0 24 24" fill="none" stroke="currentColor" strokeWidth="2" strokeLinecap="round" strokeLinejoin="round" className="lucide lucide-x">
                                      <path d="M18 6 6 18"/>
                                      <path d="m6 6 12 12"/>
                                    </svg>
                                  </Button>
                                </div>
                                <p className="text-xs text-muted-foreground">
                                  Press cancel to return to dropdown
                                </p>
                              </div>
                            )}
                            <FormMessage />
                          </FormItem>
                        );
                      }}
                    />
                    
                    {/* Station */}
                    <FormField
                      control={form.control}
                      name="station"
                      render={({ field }) => (
                        <FormItem>
                          <FormLabel>Station</FormLabel>
                          <FormControl>
                            <Input placeholder="e.g. 01" {...field} />
                          </FormControl>
                          <FormMessage />
                        </FormItem>
                      )}
                    />
                    
                    {/* Officer in Charge */}
                    <FormField
                      control={form.control}
                      name="officerInCharge"
                      render={({ field }) => (
                        <FormItem>
                          <FormLabel>Officer in Charge</FormLabel>
                          <FormControl>
                            <Input placeholder="e.g. CM Smith" {...field} />
                          </FormControl>
                          <FormMessage />
                        </FormItem>
                      )}
                    />
                    
                    {/* Watch */}
                    <FormField
                      control={form.control}
                      name="watch"
                      render={({ field }) => (
                        <FormItem>
                          <FormLabel>Watch</FormLabel>
                          <Select 
                            onValueChange={field.onChange} 
                            value={field.value}
                          >
                            <FormControl>
                              <SelectTrigger>
                                <SelectValue placeholder="Select watch" />
                              </SelectTrigger>
                            </FormControl>
                            <SelectContent>
                              {WATCH_OPTIONS.map(option => (
                                <SelectItem key={option.value} value={option.value}>
                                  {option.label}
                                </SelectItem>
                              ))}
                            </SelectContent>
                          </Select>
                          <FormMessage />
                        </FormItem>
                      )}
                    />
                  </div>
                  
                  {/* Resources Used */}
                  <div className="space-y-4">
                    <h3 className="text-md font-medium">Resources Used</h3>
                    <div className="grid grid-cols-2 md:grid-cols-3 gap-4">
                      {/* Pumps */}
                      <FormField
                        control={form.control}
                        name="pumps"
                        render={({ field }) => (
                          <FormItem>
                            <FormLabel>Pumps</FormLabel>
                            <FormControl>
                              <Input 
                                type="number" 
                                min="0" 
                                step="1" 
                                {...field} 
                                onChange={e => field.onChange(parseFloat(e.target.value) || 0)}
                                value={field.value}
                              />
                            </FormControl>
                            <FormMessage />
                          </FormItem>
                        )}
                      />
                      
                      {/* Water Carrier */}
                      <FormField
                        control={form.control}
                        name="waterCarrier"
                        render={({ field }) => (
                          <FormItem>
                            <FormLabel>Water Carrier</FormLabel>
                            <FormControl>
                              <Input 
                                type="number" 
                                min="0" 
                                step="1" 
                                {...field} 
                                onChange={e => field.onChange(parseFloat(e.target.value) || 0)}
                                value={field.value}
                              />
                            </FormControl>
                            <FormMessage />
                          </FormItem>
                        )}
                      />
                      
                      {/* Support Pump */}
                      <FormField
                        control={form.control}
                        name="supportPump"
                        render={({ field }) => (
                          <FormItem>
                            <FormLabel>Support Pump</FormLabel>
                            <FormControl>
                              <Input 
                                type="number" 
                                min="0" 
                                step="1" 
                                {...field} 
                                onChange={e => field.onChange(parseFloat(e.target.value) || 0)}
                                value={field.value}
                              />
                            </FormControl>
                            <FormMessage />
                          </FormItem>
                        )}
                      />
                      
                      {/* Aerial Ladder */}
                      <FormField
                        control={form.control}
                        name="aerialLadder"
                        render={({ field }) => (
                          <FormItem>
                            <FormLabel>Aerial Ladder Platform</FormLabel>
                            <FormControl>
                              <Input 
                                type="number" 
                                min="0" 
                                step="1" 
                                {...field} 
                                onChange={e => field.onChange(parseFloat(e.target.value) || 0)}
                                value={field.value}
                              />
                            </FormControl>
                            <FormMessage />
                          </FormItem>
                        )}
                      />
                      
                      {/* Rescue Support */}
                      <FormField
                        control={form.control}
                        name="rescueSupport"
                        render={({ field }) => (
                          <FormItem>
                            <FormLabel>Rescue Support Unit</FormLabel>
                            <FormControl>
                              <Input 
                                type="number" 
                                min="0" 
                                step="1" 
                                {...field} 
                                onChange={e => field.onChange(parseFloat(e.target.value) || 0)}
                                value={field.value}
                              />
                            </FormControl>
                            <FormMessage />
                          </FormItem>
                        )}
                      />
                      
                      {/* Incident Command */}
                      <FormField
                        control={form.control}
                        name="incidentCommand"
                        render={({ field }) => (
                          <FormItem>
                            <FormLabel>Incident Command Vehicle</FormLabel>
                            <FormControl>
                              <Input 
                                type="number" 
                                min="0" 
                                step="1" 
                                {...field} 
                                onChange={e => field.onChange(parseFloat(e.target.value) || 0)}
                                value={field.value}
                              />
                            </FormControl>
                            <FormMessage />
                          </FormItem>
                        )}
                      />
                    </div>
                  </div>
                  
                  {/* Notes */}
                  <FormField
                    control={form.control}
                    name="notes"
                    render={({ field }) => (
                      <FormItem>
                        <FormLabel>Notes</FormLabel>
                        <FormControl>
                          <Textarea 
                            placeholder="Additional information about the incident" 
                            className="min-h-[100px]"
                            {...field} 
                          />
                        </FormControl>
                        <FormMessage />
                      </FormItem>
                    )}
                  />
                  
                  {/* Safety Check */}
                  <FormField
                    control={form.control}
                    name="safetyCheck"
                    render={({ field }) => (
                      <FormItem className="flex flex-row items-start space-x-3 space-y-0">
                        <FormControl>
                          <Checkbox
                            checked={field.value}
                            onCheckedChange={field.onChange}
                          />
                        </FormControl>
                        <div className="space-y-1 leading-none">
                          <FormLabel>Safety & Wellbeing Check Required</FormLabel>
                          <FormDescription>
                            Check this box if a safety and wellbeing follow-up is required
                          </FormDescription>
                        </div>
                      </FormItem>
                    )}
                  />
                  
                  {/* Safety Check Complete */}
                  <FormField
                    control={form.control}
                    name="safetyCheckComplete"
                    render={({ field }) => (
                      <FormItem>
                        <FormLabel>Safety Check Status</FormLabel>
                        <Select 
                          onValueChange={field.onChange} 
                          value={field.value}
                        >
                          <FormControl>
                            <SelectTrigger>
                              <SelectValue placeholder="Select status" />
                            </SelectTrigger>
                          </FormControl>
                          <SelectContent>
                            <SelectItem value="N/A">Not Applicable</SelectItem>
                            <SelectItem value="Yes">Completed</SelectItem>
                            <SelectItem value="No">Not Completed</SelectItem>
                            <SelectItem value="Pending">Pending</SelectItem>
                          </SelectContent>
                        </Select>
                        <FormMessage />
                      </FormItem>
                    )}
                  />
                  
                  {/* Additional Notes */}
                  <FormField
                    control={form.control}
                    name="additionalNotes"
                    render={({ field }) => (
                      <FormItem>
                        <FormLabel>Additional Notes</FormLabel>
                        <FormControl>
                          <Textarea 
                            placeholder="Any other details or follow-up information" 
                            className="min-h-[100px]"
                            {...field} 
                          />
                        </FormControl>
                        <FormMessage />
                      </FormItem>
                    )}
                  />
                  
                  <div className="flex justify-end space-x-2">
                    <Button
                      type="button"
                      variant="outline"
                      onClick={() => navigate("/incidents")}
                    >
                      Cancel
                    </Button>
                    <Button 
                      type="submit"
                      disabled={updateMutation.isPending}
                      className="gap-2"
                    >
                      {updateMutation.isPending && (
                        <div className="animate-spin h-4 w-4 border-2 border-current border-t-transparent rounded-full" />
                      )}
                      <Save className="h-4 w-4 mr-1" />
                      Save Changes
                    </Button>
                  </div>
                </form>
              </Form>
            </CardContent>
          </Card>
        )}
      </div>
    </>
  );
}
