import React, { useState } from "react";
import { useQuery, useQueryClient } from "@tanstack/react-query";
import { Link } from "wouter";
import { Helmet } from "react-helmet";
import { DataTable } from "@/components/ui/data-table";
import { Button } from "@/components/ui/button";
import { Badge } from "@/components/ui/badge";
import { Input } from "@/components/ui/input";
import { Select, SelectContent, SelectItem, SelectTrigger, SelectValue } from "@/components/ui/select";
import { getIncidentTypeConfig, getIncidentStatus, STATUS_TYPES, INCIDENT_TYPE_OPTIONS } from "@/lib/incident-types";
import { formatDate, formatTime, formatIncidentNumber } from "@/lib/utils";
import { Plus, Edit, MoreVertical, Download, FileDown, Search, Calendar, Filter } from "lucide-react";
import { Incident } from "@shared/schema";
import { Popover, PopoverContent, PopoverTrigger } from "@/components/ui/popover";
import { Calendar as CalendarComponent } from "@/components/ui/calendar";

interface IncidentsResponse {
  data: Incident[];
  total: number;
}

export default function Incidents() {
  const [pageIndex, setPageIndex] = useState(0);
  const [pageSize, setPageSize] = useState(50);
  const [filters, setFilters] = useState<Record<string, any>>({
    incidentNumber: "",
    incidentType: "",
    date: "",
    location: "",
    town: "",
  });

  const queryClient = useQueryClient();

  // Build query string with filters
  const getQueryString = () => {
    const params = new URLSearchParams({
      page: String(pageIndex + 1),
      limit: String(pageSize),
    });

    Object.entries(filters).forEach(([key, value]) => {
      if (value) params.append(key, value);
    });

    return params.toString();
  };

  // Fetch incidents with filters but no pagination
  const { data, isLoading } = useQuery<IncidentsResponse>({
    queryKey: [`/api/incidents?${getQueryString()}`],
    refetchOnWindowFocus: true,
  });

  const handleFilterChange = (key: string, value: any) => {
    setFilters(prev => ({ ...prev, [key]: value }));
    setPageIndex(0); // Reset to first page when filters change
  };

  const resetFilters = () => {
    setFilters({
      incidentNumber: "",
      incidentType: "",
      date: "",
      location: "",
      town: "",
    });
    setPageIndex(0);
  };

  const incidentsColumns = [
    {
      accessorKey: "date",
      header: "Date",
      cell: ({ row }) => formatDate(row.original.date),
    },
    {
      accessorKey: "time",
      header: "Time",
      cell: ({ row }) => formatTime(row.original.time),
    },
    {
      accessorKey: "incidentNumber",
      header: "Incident No",
      cell: ({ row }) => (
        <span className="font-medium">
          {formatIncidentNumber(row.original.incidentNumber)}
        </span>
      ),
    },
    {
      accessorKey: "location",
      header: "Address",
      cell: ({ row }) => (
        <span className="block">
          {row.original.location}
        </span>
      ),
    },
    {
      accessorKey: "town",
      header: "Area",
      cell: ({ row }) => (
        <span className="block">
          {row.original.town || ""}
        </span>
      ),
    },
    {
      accessorKey: "station",
      header: "Stn Ground",
      cell: ({ row }) => (
        <span className="block">
          {row.original.station || ""}
        </span>
      ),
    },
    {
      accessorKey: "officerInCharge",
      header: "OIC",
      cell: ({ row }) => (
        <span className="block">
          {row.original.officerInCharge || ""}
        </span>
      ),
    },
    {
      accessorKey: "watch",
      header: "Watch",
      cell: ({ row }) => {
        const watch = row.original.watch || "";
        let bgColor = "bg-transparent";
        
        if (watch.toLowerCase().includes("green")) {
          bgColor = "bg-green-500 text-black font-medium";
        } else if (watch.toLowerCase().includes("red") && !watch.toLowerCase().includes("rds")) {
          bgColor = "bg-red-500 text-black font-medium";
        } else if (watch.toLowerCase().includes("blue")) {
          bgColor = "bg-blue-500 text-black font-medium";
        } else if (watch.toLowerCase().includes("white")) {
          bgColor = "bg-white border border-gray-300 text-black font-medium";
        } else if (watch.toLowerCase().includes("rds")) {
          bgColor = "bg-amber-500 text-black font-medium";
        }
        
        return (
          <span className={`block ${bgColor} px-2 py-1 rounded text-center`}>
            {watch}
          </span>
        );
      },
    },
    {
      accessorKey: "incidentType",
      header: "Type",
      cell: ({ row }) => {
        const typeConfig = getIncidentTypeConfig(row.original.incidentType);
        return (
          <Badge className={`incident-type-badge ${typeConfig.badge}`}>
            {typeConfig.label}
          </Badge>
        );
      },
    },

    {
      accessorKey: "notes",
      header: "Notes",
      cell: ({ row }) => (
        <span className="block">
          {row.original.notes || ""}
        </span>
      ),
    },
    {
      id: "actions",
      header: "Edit",
      cell: ({ row }) => (
        <div className="text-center">
          <Button
            variant="ghost"
            size="icon"
            asChild
          >
            <Link to={`/incidents/${row.original.id}`}>
              <Edit className="h-4 w-4 text-primary" />
              <span className="sr-only">Edit</span>
            </Link>
          </Button>
        </div>
      ),
    },
  ];

  const handlePaginationChange = (newPageIndex: number, newPageSize: number) => {
    setPageIndex(newPageIndex);
    setPageSize(newPageSize);
  };

  const filterControls = (
    <div className="flex flex-wrap gap-2">
      <Select 
        value={filters.incidentType} 
        onValueChange={(value) => handleFilterChange("incidentType", value)}
      >
        <SelectTrigger className="w-[180px]">
          <SelectValue placeholder="All Incident Types" />
        </SelectTrigger>
        <SelectContent>
          <SelectItem value="all">All Incident Types</SelectItem>
          {INCIDENT_TYPE_OPTIONS.map(option => (
            <SelectItem key={option.value} value={option.value}>
              {option.label}
            </SelectItem>
          ))}
        </SelectContent>
      </Select>
      
      <Popover>
        <PopoverTrigger asChild>
          <Button variant="outline" className="w-[180px]">
            <Calendar className="mr-2 h-4 w-4" />
            {filters.date || "Select Date"}
          </Button>
        </PopoverTrigger>
        <PopoverContent className="w-auto p-0">
          <CalendarComponent
            mode="single"
            selected={filters.date ? new Date(filters.date) : undefined}
            onSelect={(date) => handleFilterChange("date", date ? date.toISOString().split('T')[0] : "")}
            initialFocus
          />
        </PopoverContent>
      </Popover>
      
      <Input
        placeholder="Town/City"
        value={filters.town}
        onChange={(e) => handleFilterChange("town", e.target.value)}
        className="w-[180px]"
      />
      
      <Button variant="ghost" onClick={resetFilters}>
        Reset Filters
      </Button>
    </div>
  );

  return (
    <>
      <Helmet>
        <title>Incidents | Emergency Response Incident Management System</title>
        <meta name="description" content="View, search and manage emergency response incidents" />
      </Helmet>
      
      <div className="p-4 md:p-6">
        <div className="mb-6">
          <div className="flex flex-col md:flex-row md:items-center md:justify-between gap-4">
            <div>
              <h1 className="text-2xl font-bold text-neutral-800">Incidents</h1>
              <p className="text-neutral-600">View and manage all incident records</p>
            </div>
            <div className="flex flex-wrap gap-2">
              <Button className="inline-flex items-center" asChild>
                <Link href="/incidents/new">
                  <Plus className="mr-2 h-4 w-4" />
                  New Incident
                </Link>
              </Button>
              <Button
                variant="outline"
                className="inline-flex items-center text-primary"
                asChild
              >
                <Link href="/api/export/csv">
                  <FileDown className="mr-2 h-4 w-4" />
                  Export
                </Link>
              </Button>
            </div>
          </div>
        </div>

        {/* Incidents Table */}
        <div className="bg-white rounded-lg shadow-sm border border-neutral-200 overflow-hidden w-full max-w-full">
          <div className="p-4 border-b border-neutral-200">
            <div className="flex flex-col md:flex-row md:items-center md:justify-between gap-4">
              <div className="relative">
                <Input
                  placeholder="Search incidents..."
                  value={filters.incidentNumber}
                  onChange={(e) => handleFilterChange("incidentNumber", e.target.value)}
                  className="pl-3 h-10 w-[200px]"
                />
              </div>
              {filterControls}
            </div>
          </div>
          
          <div className="relative">
            {isLoading && (
              <div className="absolute inset-0 bg-white/80 flex items-center justify-center z-10">
                <div className="animate-spin h-8 w-8 border-4 border-primary border-t-transparent rounded-full"></div>
              </div>
            )}
            
            <div className="overflow-x-auto">
              <table className="w-full caption-bottom text-sm text-center">
                <thead className="bg-neutral-100">
                  <tr>
                    <th className="p-4 font-medium text-gray-600">Call No</th>
                    <th className="p-4 font-medium text-gray-600">Date</th>
                    <th className="p-4 font-medium text-gray-600">Time</th>
                    <th className="p-4 font-medium text-gray-600">Incident No</th>
                    <th className="p-4 font-medium text-gray-600">Address</th>
                    <th className="p-4 font-medium text-gray-600">Area</th>
                    <th className="p-4 font-medium text-gray-600">Stn Ground</th>
                    <th className="p-4 font-medium text-gray-600">OIC</th>
                    <th className="p-4 font-medium text-gray-600">Watch</th>
                    <th className="p-4 font-medium text-gray-600">Type</th>
                    <th className="p-4 font-medium text-gray-600">Notes</th>
                    <th className="p-4 font-medium text-gray-600">Edit</th>
                  </tr>
                </thead>
                <tbody>
                  {!data?.data || data.data.length === 0 ? (
                    <tr>
                      <td colSpan={12} className="h-24 text-center">
                        No results found.
                      </td>
                    </tr>
                  ) : (
                    data.data.map((incident, index) => (
                      <tr key={incident.id} className="border-b hover:bg-neutral-50">
                        <td className="p-2 text-center">{index + 1}</td>
                        <td className="p-2 text-center">{formatDate(incident.date)}</td>
                        <td className="p-2 text-center">{formatTime(incident.time)}</td>
                        <td className="p-2 text-center font-medium">{formatIncidentNumber(incident.incidentNumber)}</td>
                        <td className="p-2 text-center">{incident.location}</td>
                        <td className="p-2 text-center">{incident.town || "-"}</td>
                        <td className="p-2 text-center">{incident.station || "-"}</td>
                        <td className="p-2 text-center">{incident.officerInCharge || "-"}</td>
                        <td className="p-2 text-center">
                          {incident.watch ? (
                            <span className={`block px-2 py-1 rounded text-center ${
                              incident.watch.toLowerCase().includes("green") ? "bg-green-500 text-black font-medium" :
                              incident.watch.toLowerCase().includes("red") && !incident.watch.toLowerCase().includes("rds") ? "bg-red-500 text-black font-medium" :
                              incident.watch.toLowerCase().includes("blue") ? "bg-blue-500 text-black font-medium" :
                              incident.watch.toLowerCase().includes("white") ? "bg-white border border-gray-300 text-black font-medium" :
                              incident.watch.toLowerCase().includes("rds") ? "bg-amber-500 text-black font-medium" :
                              "bg-transparent"
                            }`}>
                              {incident.watch}
                            </span>
                          ) : "-"}
                        </td>
                        <td className="p-2 text-center">
                          {incident.incidentType ? (
                            <Badge className={`incident-type-badge ${getIncidentTypeConfig(incident.incidentType).badge}`}>
                              {getIncidentTypeConfig(incident.incidentType).label}
                            </Badge>
                          ) : "-"}
                        </td>
                        <td className="p-2 text-center">{incident.notes || "-"}</td>
                        <td className="p-2 text-center">
                          <Button size="sm" variant="ghost" asChild>
                            <Link href={`/incidents/${incident.id}`}>
                              <Edit className="h-4 w-4" />
                            </Link>
                          </Button>
                        </td>
                      </tr>
                    ))
                  )}
                </tbody>
              </table>
            </div>
          </div>
          
          <div className="p-4 border-t border-neutral-200">
            <div className="flex items-center gap-2">
              <p className="text-sm text-neutral-600">
                Showing all <span className="font-medium">{data?.data?.length || 0}</span> incidents
              </p>
            </div>
          </div>
        </div>
      </div>
    </>
  );
}
