import React, { createContext, useState, useContext, useEffect } from 'react';

interface SettingsContextType {
  dateFormat: string;
  setDateFormat: (format: string) => void;
  timeFormat: string;
  setTimeFormat: (format: string) => void;
  darkMode: boolean;
  setDarkMode: (enabled: boolean) => void;
  autoSave: boolean;
  setAutoSave: (enabled: boolean) => void;
  incidentNumberPrefix: string;
  setIncidentNumberPrefix: (prefix: string) => void;
  saveSettings: () => void;
}

const defaultSettings = {
  dateFormat: 'dd-mm-yyyy', // Standardized to DD-MM-YYYY
  timeFormat: '24h',
  darkMode: false,
  autoSave: true,
  incidentNumberPrefix: 'HH',
};

const SettingsContext = createContext<SettingsContextType | undefined>(undefined);

export function SettingsProvider({ children }: { children: React.ReactNode }) {
  // Initialize state from localStorage if available
  const [dateFormat, setDateFormat] = useState(() => {
    const saved = localStorage.getItem('settings_dateFormat');
    return saved || defaultSettings.dateFormat;
  });
  
  const [timeFormat, setTimeFormat] = useState(() => {
    const saved = localStorage.getItem('settings_timeFormat');
    return saved || defaultSettings.timeFormat;
  });
  
  const [darkMode, setDarkMode] = useState(() => {
    const saved = localStorage.getItem('settings_darkMode');
    return saved === 'true' || defaultSettings.darkMode;
  });
  
  const [autoSave, setAutoSave] = useState(() => {
    const saved = localStorage.getItem('settings_autoSave');
    return saved === 'true' || defaultSettings.autoSave;
  });
  
  const [incidentNumberPrefix, setIncidentNumberPrefix] = useState(() => {
    const saved = localStorage.getItem('settings_incidentNumberPrefix');
    return saved || defaultSettings.incidentNumberPrefix;
  });

  // Save all settings to localStorage
  const saveSettings = () => {
    localStorage.setItem('settings_dateFormat', dateFormat);
    localStorage.setItem('settings_timeFormat', timeFormat);
    localStorage.setItem('settings_darkMode', String(darkMode));
    localStorage.setItem('settings_autoSave', String(autoSave));
    localStorage.setItem('settings_incidentNumberPrefix', incidentNumberPrefix);
  };

  return (
    <SettingsContext.Provider
      value={{
        dateFormat,
        setDateFormat,
        timeFormat,
        setTimeFormat,
        darkMode,
        setDarkMode,
        autoSave,
        setAutoSave,
        incidentNumberPrefix,
        setIncidentNumberPrefix,
        saveSettings,
      }}
    >
      {children}
    </SettingsContext.Provider>
  );
}

export function useSettings() {
  const context = useContext(SettingsContext);
  if (context === undefined) {
    throw new Error('useSettings must be used within a SettingsProvider');
  }
  return context;
}