import React, { useState, useRef } from 'react';
import { Helmet } from 'react-helmet';
import { Button } from '@/components/ui/button';
import { Card, CardContent, CardHeader, CardTitle } from '@/components/ui/card';
import { Textarea } from '@/components/ui/textarea';
import { useToast } from '@/hooks/use-toast';
import { queryClient } from '@/lib/queryClient';
import { useLocation } from 'wouter';
import { Tabs, TabsContent, TabsList, TabsTrigger } from '@/components/ui/tabs';
import { Input } from '@/components/ui/input';
import { Upload, File, Clipboard } from 'lucide-react';

export default function ImportData() {
  const [csvData, setCsvData] = useState('');
  const [isLoading, setIsLoading] = useState(false);
  const [activeTab, setActiveTab] = useState('paste');
  const fileInputRef = useRef<HTMLInputElement>(null);
  const { toast } = useToast();
  const [_, navigate] = useLocation();

  const handleImport = async () => {
    if (activeTab === 'paste' && !csvData.trim()) {
      toast({
        title: 'Error',
        description: 'Please paste CSV data before importing',
        variant: 'destructive',
      });
      return;
    }

    if (activeTab === 'upload' && !fileInputRef.current?.files?.length) {
      toast({
        title: 'Error',
        description: 'Please select a CSV file to upload',
        variant: 'destructive',
      });
      return;
    }

    setIsLoading(true);
    try {
      let response;
      
      if (activeTab === 'paste') {
        // Import via pasted CSV data
        response = await fetch('/api/import/csv', {
          method: 'POST',
          headers: { 'Content-Type': 'application/json' },
          body: JSON.stringify({ csvData }),
        });
      } else {
        // Import via file upload
        const formData = new FormData();
        if (fileInputRef.current?.files?.[0]) {
          formData.append('csvFile', fileInputRef.current.files[0]);
          
          response = await fetch('/api/import/csv-file', {
            method: 'POST',
            body: formData,
          });
        } else {
          throw new Error('No file selected');
        }
      }

      if (response && response.ok) {
        toast({
          title: 'Success',
          description: 'Data imported successfully',
        });
        
        // Invalidate queries to refresh data
        queryClient.invalidateQueries({ queryKey: ['/api/incidents'] });
        queryClient.invalidateQueries({ queryKey: ['/api/stats'] });
        
        navigate('/incidents');
      } else if (response) {
        const error = await response.json();
        throw new Error(error.message || 'Failed to import data');
      }
    } catch (error) {
      toast({
        title: 'Import Failed',
        description: error instanceof Error ? error.message : 'An unknown error occurred',
        variant: 'destructive',
      });
    } finally {
      setIsLoading(false);
    }
  };

  return (
    <>
      <Helmet>
        <title>Import Data | Emergency Response Incident Management System</title>
        <meta name="description" content="Import incident data from CSV format" />
      </Helmet>

      <div className="p-4 md:p-6">
        <div className="mb-6">
          <h1 className="text-2xl font-bold text-neutral-800">Import Incident Data</h1>
          <p className="text-neutral-600">Add incident data from CSV files</p>
        </div>

        <Card>
          <CardHeader>
            <CardTitle>CSV Data Import</CardTitle>
          </CardHeader>
          <CardContent>
            <Tabs value={activeTab} onValueChange={setActiveTab} className="space-y-4">
              <TabsList className="grid w-full grid-cols-2">
                <TabsTrigger value="paste" className="flex items-center gap-2">
                  <Clipboard className="h-4 w-4" />
                  <span>Paste CSV</span>
                </TabsTrigger>
                <TabsTrigger value="upload" className="flex items-center gap-2">
                  <Upload className="h-4 w-4" />
                  <span>Upload File</span>
                </TabsTrigger>
              </TabsList>
              
              <TabsContent value="paste" className="space-y-4">
                <p className="text-sm text-neutral-600">
                  Copy data from your spreadsheet and paste it below. Ensure the first row contains column headers.
                </p>
                
                <Textarea 
                  value={csvData}
                  onChange={e => setCsvData(e.target.value)}
                  placeholder="Paste CSV data here..."
                  className="min-h-[300px] font-mono text-sm"
                />
              </TabsContent>
              
              <TabsContent value="upload" className="space-y-4">
                <p className="text-sm text-neutral-600">
                  Select a CSV file to upload. The file should have column headers in the first row.
                </p>
                
                <div className="grid w-full items-center gap-1.5">
                  <label htmlFor="csvFile" className="text-sm font-medium">
                    CSV File
                  </label>
                  <Input
                    id="csvFile"
                    type="file"
                    accept=".csv"
                    ref={fileInputRef}
                    className="cursor-pointer"
                  />
                  <p className="text-xs text-muted-foreground">
                    Accepted format: .csv
                  </p>
                </div>
              </TabsContent>
              
              <div className="flex justify-end pt-4 border-t">
                <Button 
                  onClick={handleImport}
                  disabled={isLoading}
                  className="flex items-center gap-2"
                >
                  {isLoading ? 'Importing...' : 'Import Data'}
                  {!isLoading && <File className="h-4 w-4" />}
                </Button>
              </div>
            </Tabs>
          </CardContent>
        </Card>
      </div>
    </>
  );
}