import React from "react";
import { useForm } from "react-hook-form";
import { zodResolver } from "@hookform/resolvers/zod";
import { useQuery, useMutation, useQueryClient } from "@tanstack/react-query";
import { useLocation } from "wouter";
import { Helmet } from "react-helmet";
import { apiRequest } from "@/lib/queryClient";
import { insertIncidentSchema } from "@shared/schema";
import { Button } from "@/components/ui/button";
import { Input } from "@/components/ui/input";
import { Textarea } from "@/components/ui/textarea";
import { Checkbox } from "@/components/ui/checkbox";
import { Card, CardContent, CardHeader, CardTitle } from "@/components/ui/card";
import { Form, FormControl, FormDescription, FormField, FormItem, FormLabel, FormMessage } from "@/components/ui/form";
import { Select, SelectContent, SelectItem, SelectTrigger, SelectValue } from "@/components/ui/select";
import { toast } from "@/hooks/use-toast";
import { INCIDENT_TYPE_OPTIONS, WATCH_OPTIONS } from "@/lib/incident-types";
import { COMMON_AREAS } from "@/lib/utils";
import { ArrowLeft, Save } from "lucide-react";

export default function NewIncident() {
  const [_, navigate] = useLocation();
  const queryClient = useQueryClient();
  
  // Get current date and time for default values
  const now = new Date();
  const currentDate = now.toISOString().split('T')[0];
  const currentTime = now.toTimeString().slice(0, 5);
  
  // Set up form with zod resolver
  const form = useForm({
    resolver: zodResolver(insertIncidentSchema),
    defaultValues: {
      incidentNumber: "",
      date: currentDate,
      time: currentTime,
      reference: "",
      location: "",
      town: "",
      station: "",
      officerInCharge: "",
      watch: "",
      pumps: 0,
      waterCarrier: 0,
      supportPump: 0,
      aerialLadder: 0,
      rescueSupport: 0,
      incidentCommand: 0,
      incidentType: "",
      notes: "",
      safetyCheck: false,
      additionalNotes: "",
      safetyCheckComplete: "",
    },
  });
  
  // Set up mutation
  const mutation = useMutation({
    mutationFn: async (values: any) => {
      const res = await apiRequest("POST", "/api/incidents", values);
      return res.json();
    },
    onSuccess: () => {
      queryClient.invalidateQueries({ queryKey: ["/api/incidents"] });
      queryClient.invalidateQueries({ queryKey: ["/api/stats"] });
      toast({
        title: "Incident created",
        description: "The incident has been successfully created.",
      });
      navigate("/incidents");
    },
    onError: (error: any) => {
      toast({
        title: "Error",
        description: error.message || "An error occurred while creating the incident.",
        variant: "destructive",
      });
    },
  });
  
  // Form submission
  const onSubmit = (values: any) => {
    mutation.mutate(values);
  };
  
  return (
    <>
      <Helmet>
        <title>New Incident | Emergency Response Incident Management System</title>
        <meta name="description" content="Create a new emergency response incident record" />
      </Helmet>
      
      <div className="p-4 md:p-6">
        <div className="mb-6">
          <div className="flex flex-col md:flex-row md:items-center md:justify-between gap-4">
            <div className="flex items-center">
              <Button variant="ghost" className="mr-2" onClick={() => navigate(-1)}>
                <ArrowLeft className="h-4 w-4" />
              </Button>
              <div>
                <h1 className="text-2xl font-bold text-neutral-800">New Incident</h1>
                <p className="text-neutral-600">Create a new incident record</p>
              </div>
            </div>
          </div>
        </div>
        
        <Card className="max-w-4xl mx-auto">
          <CardContent className="p-6">
            <Form {...form}>
              <form onSubmit={form.handleSubmit(onSubmit)} className="space-y-6">
                <div className="grid grid-cols-1 md:grid-cols-2 gap-6">
                  {/* Incident ID */}
                  <FormField
                    control={form.control}
                    name="incidentNumber"
                    render={({ field }) => (
                      <FormItem>
                        <FormLabel>Incident ID</FormLabel>
                        <FormControl>
                          <Input placeholder="e.g. HH032473" {...field} />
                        </FormControl>
                        <FormMessage />
                      </FormItem>
                    )}
                  />
                  
                  {/* Date */}
                  <FormField
                    control={form.control}
                    name="date"
                    render={({ field }) => (
                      <FormItem>
                        <FormLabel>Date</FormLabel>
                        <FormControl>
                          <Input type="date" {...field} />
                        </FormControl>
                        <FormMessage />
                      </FormItem>
                    )}
                  />
                  
                  {/* Time */}
                  <FormField
                    control={form.control}
                    name="time"
                    render={({ field }) => (
                      <FormItem>
                        <FormLabel>Time</FormLabel>
                        <FormControl>
                          <Input type="time" {...field} />
                        </FormControl>
                        <FormMessage />
                      </FormItem>
                    )}
                  />
                  
                  {/* Incident Type */}
                  <FormField
                    control={form.control}
                    name="incidentType"
                    render={({ field }) => (
                      <FormItem>
                        <FormLabel>Incident Type</FormLabel>
                        <Select 
                          onValueChange={field.onChange} 
                          defaultValue={field.value}
                        >
                          <FormControl>
                            <SelectTrigger>
                              <SelectValue placeholder="Select incident type" />
                            </SelectTrigger>
                          </FormControl>
                          <SelectContent>
                            {INCIDENT_TYPE_OPTIONS.map(option => (
                              <SelectItem key={option.value} value={option.value}>
                                {option.label}
                              </SelectItem>
                            ))}
                          </SelectContent>
                        </Select>
                        <FormMessage />
                      </FormItem>
                    )}
                  />
                  
                  {/* Location */}
                  <FormField
                    control={form.control}
                    name="location"
                    render={({ field }) => (
                      <FormItem className="md:col-span-2">
                        <FormLabel>Location</FormLabel>
                        <FormControl>
                          <Input placeholder="Address or location details" {...field} />
                        </FormControl>
                        <FormMessage />
                      </FormItem>
                    )}
                  />
                  
                  {/* Town/City */}
                  <FormField
                    control={form.control}
                    name="town"
                    render={({ field }) => {
                      // Use local state to manage custom input
                      const [isCustom, setIsCustom] = React.useState(
                        field.value && !COMMON_AREAS.some(area => area.value === field.value) && field.value !== "other"
                      );
                      const [customValue, setCustomValue] = React.useState(
                        isCustom ? field.value : ""
                      );
                      
                      return (
                        <FormItem>
                          <FormLabel>Town / Area</FormLabel>
                          {!isCustom ? (
                            <Select 
                              onValueChange={(value) => {
                                if (value === "other") {
                                  setIsCustom(true);
                                } else {
                                  field.onChange(value);
                                }
                              }} 
                              value={field.value || ""}
                            >
                              <FormControl>
                                <SelectTrigger>
                                  <SelectValue placeholder="Select area" />
                                </SelectTrigger>
                              </FormControl>
                              <SelectContent>
                                {COMMON_AREAS.map(option => (
                                  <SelectItem key={option.value} value={option.value}>
                                    {option.label}
                                  </SelectItem>
                                ))}
                                <SelectItem value="other">Other (Custom)</SelectItem>
                              </SelectContent>
                            </Select>
                          ) : (
                            <div className="flex flex-col gap-2">
                              <div className="flex gap-2">
                                <Input
                                  placeholder="Enter custom area"
                                  value={customValue}
                                  onChange={(e) => {
                                    const newValue = e.target.value;
                                    setCustomValue(newValue);
                                    field.onChange(newValue);
                                  }}
                                  autoFocus
                                />
                                <Button 
                                  type="button" 
                                  variant="outline" 
                                  size="icon"
                                  onClick={() => {
                                    setIsCustom(false);
                                    setCustomValue("");
                                    field.onChange(""); // Reset to empty so placeholder shows
                                  }}
                                >
                                  <svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" viewBox="0 0 24 24" fill="none" stroke="currentColor" strokeWidth="2" strokeLinecap="round" strokeLinejoin="round" className="lucide lucide-x">
                                    <path d="M18 6 6 18"/>
                                    <path d="m6 6 12 12"/>
                                  </svg>
                                </Button>
                              </div>
                              <p className="text-xs text-muted-foreground">
                                Press cancel to return to dropdown
                              </p>
                            </div>
                          )}
                          <FormMessage />
                        </FormItem>
                      );
                    }}
                  />
                  
                  {/* Station */}
                  <FormField
                    control={form.control}
                    name="station"
                    render={({ field }) => (
                      <FormItem>
                        <FormLabel>Station</FormLabel>
                        <FormControl>
                          <Input placeholder="e.g. 01" {...field} />
                        </FormControl>
                        <FormMessage />
                      </FormItem>
                    )}
                  />
                  
                  {/* Officer in Charge */}
                  <FormField
                    control={form.control}
                    name="officerInCharge"
                    render={({ field }) => (
                      <FormItem>
                        <FormLabel>Officer in Charge</FormLabel>
                        <FormControl>
                          <Input placeholder="e.g. CM Smith" {...field} />
                        </FormControl>
                        <FormMessage />
                      </FormItem>
                    )}
                  />
                  
                  {/* Watch */}
                  <FormField
                    control={form.control}
                    name="watch"
                    render={({ field }) => (
                      <FormItem>
                        <FormLabel>Watch</FormLabel>
                        <Select 
                          onValueChange={field.onChange} 
                          defaultValue={field.value}
                        >
                          <FormControl>
                            <SelectTrigger>
                              <SelectValue placeholder="Select watch" />
                            </SelectTrigger>
                          </FormControl>
                          <SelectContent>
                            {WATCH_OPTIONS.map(option => (
                              <SelectItem key={option.value} value={option.value}>
                                {option.label}
                              </SelectItem>
                            ))}
                          </SelectContent>
                        </Select>
                        <FormMessage />
                      </FormItem>
                    )}
                  />
                </div>
                
                {/* Resources Used */}
                <div className="space-y-4">
                  <h3 className="text-md font-medium">Resources Used</h3>
                  <div className="grid grid-cols-2 md:grid-cols-3 gap-4">
                    {/* Pumps */}
                    <FormField
                      control={form.control}
                      name="pumps"
                      render={({ field }) => (
                        <FormItem>
                          <FormLabel>Pumps</FormLabel>
                          <FormControl>
                            <Input 
                              type="number" 
                              min="0" 
                              step="1" 
                              {...field} 
                              onChange={e => field.onChange(parseInt(e.target.value) || 0)}
                            />
                          </FormControl>
                          <FormMessage />
                        </FormItem>
                      )}
                    />
                    
                    {/* Water Carrier */}
                    <FormField
                      control={form.control}
                      name="waterCarrier"
                      render={({ field }) => (
                        <FormItem>
                          <FormLabel>Water Carrier</FormLabel>
                          <FormControl>
                            <Input 
                              type="number" 
                              min="0" 
                              step="1" 
                              {...field} 
                              onChange={e => field.onChange(parseInt(e.target.value) || 0)}
                            />
                          </FormControl>
                          <FormMessage />
                        </FormItem>
                      )}
                    />
                    
                    {/* Support Pump */}
                    <FormField
                      control={form.control}
                      name="supportPump"
                      render={({ field }) => (
                        <FormItem>
                          <FormLabel>Support Pump</FormLabel>
                          <FormControl>
                            <Input 
                              type="number" 
                              min="0" 
                              step="1" 
                              {...field} 
                              onChange={e => field.onChange(parseInt(e.target.value) || 0)}
                            />
                          </FormControl>
                          <FormMessage />
                        </FormItem>
                      )}
                    />
                    
                    {/* Aerial Ladder */}
                    <FormField
                      control={form.control}
                      name="aerialLadder"
                      render={({ field }) => (
                        <FormItem>
                          <FormLabel>Aerial Ladder Platform</FormLabel>
                          <FormControl>
                            <Input 
                              type="number" 
                              min="0" 
                              step="1" 
                              {...field} 
                              onChange={e => field.onChange(parseInt(e.target.value) || 0)}
                            />
                          </FormControl>
                          <FormMessage />
                        </FormItem>
                      )}
                    />
                    
                    {/* Rescue Support */}
                    <FormField
                      control={form.control}
                      name="rescueSupport"
                      render={({ field }) => (
                        <FormItem>
                          <FormLabel>Rescue Support Unit</FormLabel>
                          <FormControl>
                            <Input 
                              type="number" 
                              min="0" 
                              step="1" 
                              {...field} 
                              onChange={e => field.onChange(parseInt(e.target.value) || 0)}
                            />
                          </FormControl>
                          <FormMessage />
                        </FormItem>
                      )}
                    />
                    
                    {/* Incident Command */}
                    <FormField
                      control={form.control}
                      name="incidentCommand"
                      render={({ field }) => (
                        <FormItem>
                          <FormLabel>Incident Command Vehicle</FormLabel>
                          <FormControl>
                            <Input 
                              type="number" 
                              min="0" 
                              step="1" 
                              {...field} 
                              onChange={e => field.onChange(parseInt(e.target.value) || 0)}
                            />
                          </FormControl>
                          <FormMessage />
                        </FormItem>
                      )}
                    />
                  </div>
                </div>
                
                {/* Notes */}
                <FormField
                  control={form.control}
                  name="notes"
                  render={({ field }) => (
                    <FormItem>
                      <FormLabel>Notes</FormLabel>
                      <FormControl>
                        <Textarea 
                          placeholder="Additional information about the incident" 
                          className="min-h-[100px]"
                          {...field} 
                        />
                      </FormControl>
                      <FormMessage />
                    </FormItem>
                  )}
                />
                
                {/* Safety Check */}
                <FormField
                  control={form.control}
                  name="safetyCheck"
                  render={({ field }) => (
                    <FormItem className="flex flex-row items-start space-x-3 space-y-0">
                      <FormControl>
                        <Checkbox
                          checked={field.value}
                          onCheckedChange={field.onChange}
                        />
                      </FormControl>
                      <div className="space-y-1 leading-none">
                        <FormLabel>Safety & Wellbeing Check Required</FormLabel>
                        <FormDescription>
                          Check this box if a safety and wellbeing follow-up is required
                        </FormDescription>
                      </div>
                    </FormItem>
                  )}
                />
                
                {/* Additional Notes */}
                <FormField
                  control={form.control}
                  name="additionalNotes"
                  render={({ field }) => (
                    <FormItem>
                      <FormLabel>Additional Notes</FormLabel>
                      <FormControl>
                        <Textarea 
                          placeholder="Any other details or follow-up information" 
                          className="min-h-[100px]"
                          {...field} 
                        />
                      </FormControl>
                      <FormMessage />
                    </FormItem>
                  )}
                />
                
                <div className="flex justify-end space-x-2">
                  <Button
                    type="button"
                    variant="outline"
                    onClick={() => navigate("/incidents")}
                  >
                    Cancel
                  </Button>
                  <Button 
                    type="submit"
                    disabled={mutation.isPending}
                    className="gap-2"
                  >
                    {mutation.isPending && (
                      <div className="animate-spin h-4 w-4 border-2 border-current border-t-transparent rounded-full" />
                    )}
                    <Save className="h-4 w-4 mr-1" />
                    Save Incident
                  </Button>
                </div>
              </form>
            </Form>
          </CardContent>
        </Card>
      </div>
    </>
  );
}
