import React, { useState } from "react";
import { useQuery } from "@tanstack/react-query";
import { Helmet } from "react-helmet";
import { 
  Card, 
  CardContent, 
  CardHeader, 
  CardTitle,
  CardDescription
} from "@/components/ui/card";
import { Button } from "@/components/ui/button";
import { Select, SelectContent, SelectItem, SelectTrigger, SelectValue } from "@/components/ui/select";
import { 
  BarChart, 
  Bar, 
  LineChart, 
  Line, 
  PieChart, 
  Pie, 
  Cell, 
  XAxis, 
  YAxis, 
  CartesianGrid, 
  Tooltip, 
  Legend, 
  ResponsiveContainer 
} from "recharts";
import { Stats } from "@shared/schema";
import { FileDown, Filter, BarChart3, PieChart as PieChartIcon, TrendingUp } from "lucide-react";
import { getIncidentTypeConfig } from "@/lib/incident-types";

type ChartType = 'bar' | 'pie' | 'line';
type DataGrouping = 'type' | 'town' | 'month';

export default function Reports() {
  const [chartType, setChartType] = useState<ChartType>('bar');
  const [dataGrouping, setDataGrouping] = useState<DataGrouping>('type');
  const [timeRange, setTimeRange] = useState<string>("all");

  // Fetch dashboard stats
  const { data: stats, isLoading } = useQuery<Stats>({
    queryKey: ["/api/stats"],
  });

  // Prepare chart data based on selected grouping
  const getChartData = () => {
    if (!stats) return [];

    switch (dataGrouping) {
      case 'type':
        return Object.entries(stats.byType || {})
          .map(([name, value]) => ({ name, value }))
          .sort((a, b) => b.value - a.value);
      case 'town':
        return Object.entries(stats.byTown || {})
          .map(([name, value]) => ({ name, value }))
          .sort((a, b) => b.value - a.value);
      case 'month':
        const months = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"];
        return Object.entries(stats.byMonth || {})
          .map(([name, value]) => ({ name, value }))
          .sort((a, b) => {
            if (a.name === "Unknown") return 1;
            if (b.name === "Unknown") return -1;
            return months.indexOf(a.name) - months.indexOf(b.name);
          });
      default:
        return [];
    }
  };

  const chartData = getChartData();

  // Colors for charts
  const COLORS = ['hsl(var(--chart-1))', 'hsl(var(--chart-2))', 'hsl(var(--chart-3))', 'hsl(var(--chart-4))', 'hsl(var(--chart-5))'];

  // Custom formatter for tooltips
  const tooltipFormatter = (value: number) => [`${value} incidents`, 'Count'];

  // Render the appropriate chart type
  const renderChart = () => {
    const height = dataGrouping === 'type' && chartData.length > 8 ? 400 : 300;

    switch (chartType) {
      case 'bar':
        return (
          <ResponsiveContainer width="100%" height={height}>
            <BarChart
              data={chartData}
              margin={{ top: 20, right: 30, left: 20, bottom: 70 }}
            >
              <CartesianGrid strokeDasharray="3 3" vertical={false} />
              <XAxis 
                dataKey="name" 
                angle={-45} 
                textAnchor="end" 
                height={70} 
                tick={{ fontSize: 12 }}
              />
              <YAxis />
              <Tooltip formatter={tooltipFormatter} />
              <Legend />
              <Bar dataKey="value" fill="hsl(var(--chart-1))" name="Incidents" />
            </BarChart>
          </ResponsiveContainer>
        );
      
      case 'pie':
        return (
          <ResponsiveContainer width="100%" height={height}>
            <PieChart>
              <Pie
                data={chartData}
                cx="50%"
                cy="50%"
                labelLine={true}
                outerRadius={120}
                fill="#8884d8"
                dataKey="value"
                label={({ name, percent }) => `${name}: ${(percent * 100).toFixed(0)}%`}
              >
                {chartData.map((_, index) => (
                  <Cell key={`cell-${index}`} fill={COLORS[index % COLORS.length]} />
                ))}
              </Pie>
              <Tooltip formatter={tooltipFormatter} />
              <Legend />
            </PieChart>
          </ResponsiveContainer>
        );
      
      case 'line':
        return (
          <ResponsiveContainer width="100%" height={height}>
            <LineChart
              data={chartData}
              margin={{ top: 20, right: 30, left: 20, bottom: 70 }}
            >
              <CartesianGrid strokeDasharray="3 3" />
              <XAxis 
                dataKey="name" 
                angle={-45} 
                textAnchor="end" 
                height={70} 
                tick={{ fontSize: 12 }}
              />
              <YAxis />
              <Tooltip formatter={tooltipFormatter} />
              <Legend />
              <Line 
                type="monotone" 
                dataKey="value" 
                stroke="hsl(var(--chart-1))" 
                activeDot={{ r: 8 }} 
                name="Incidents"
              />
            </LineChart>
          </ResponsiveContainer>
        );
    }
  };

  // Get additional metrics
  const getMetrics = () => {
    if (!stats) return [];

    const totalIncidents = stats.totalIncidents || 0;
    
    switch (dataGrouping) {
      case 'type':
        const typeData = Object.entries(stats.byType || {})
          .sort((a, b) => b[1] - a[1]);
        
        return [
          {
            title: "Most Common Incident",
            value: typeData[0] ? typeData[0][0] : "None",
            percentage: typeData[0] ? `${((typeData[0][1] / totalIncidents) * 100).toFixed(1)}%` : "0%"
          },
          {
            title: "Least Common Incident",
            value: typeData[typeData.length - 1] ? typeData[typeData.length - 1][0] : "None",
            percentage: typeData[typeData.length - 1] ? `${((typeData[typeData.length - 1][1] / totalIncidents) * 100).toFixed(1)}%` : "0%"
          }
        ];
      
      case 'town':
        const townData = Object.entries(stats.byTown || {})
          .sort((a, b) => b[1] - a[1]);
        
        return [
          {
            title: "Most Active Location",
            value: townData[0] ? townData[0][0] : "None",
            percentage: townData[0] ? `${((townData[0][1] / totalIncidents) * 100).toFixed(1)}%` : "0%"
          },
          {
            title: "Least Active Location",
            value: townData[townData.length - 1] ? townData[townData.length - 1][0] : "None",
            percentage: townData[townData.length - 1] ? `${((townData[townData.length - 1][1] / totalIncidents) * 100).toFixed(1)}%` : "0%"
          }
        ];
      
      case 'month':
        const monthData = Object.entries(stats.byMonth || {})
          .filter(([name]) => name !== "Unknown")
          .sort((a, b) => b[1] - a[1]);
        
        return [
          {
            title: "Busiest Month",
            value: monthData[0] ? monthData[0][0] : "None",
            percentage: monthData[0] ? `${((monthData[0][1] / totalIncidents) * 100).toFixed(1)}%` : "0%"
          },
          {
            title: "Quietest Month",
            value: monthData[monthData.length - 1] ? monthData[monthData.length - 1][0] : "None",
            percentage: monthData[monthData.length - 1] ? `${((monthData[monthData.length - 1][1] / totalIncidents) * 100).toFixed(1)}%` : "0%"
          }
        ];
    }
  };

  const metrics = getMetrics();

  return (
    <>
      <Helmet>
        <title>Reports | Emergency Response Incident Management System</title>
        <meta name="description" content="Analyze emergency response incident data with powerful visualization tools" />
      </Helmet>
      
      <div className="p-4 md:p-6">
        <div className="mb-6">
          <div className="flex flex-col md:flex-row md:items-center md:justify-between gap-4">
            <div>
              <h1 className="text-2xl font-bold text-neutral-800">Reports</h1>
              <p className="text-neutral-600">Analyze incident data with charts and graphs</p>
            </div>
            <div>
              <Button
                variant="outline"
                className="inline-flex items-center text-primary"
                asChild
              >
                <a href="/api/export/csv">
                  <FileDown className="mr-2 h-4 w-4" />
                  Export Data
                </a>
              </Button>
            </div>
          </div>
        </div>

        {/* Chart Controls */}
        <Card className="mb-6">
          <CardHeader>
            <CardTitle>Report Configuration</CardTitle>
            <CardDescription>Customize how the data is presented</CardDescription>
          </CardHeader>
          <CardContent>
            <div className="flex flex-wrap gap-4">
              <div className="space-y-2">
                <label className="text-sm font-medium">Chart Type</label>
                <div className="flex space-x-2">
                  <Button 
                    variant={chartType === 'bar' ? 'default' : 'outline'} 
                    size="sm"
                    onClick={() => setChartType('bar')}
                    className="gap-2"
                  >
                    <BarChart3 className="h-4 w-4" />
                    Bar
                  </Button>
                  <Button 
                    variant={chartType === 'pie' ? 'default' : 'outline'} 
                    size="sm"
                    onClick={() => setChartType('pie')}
                    className="gap-2"
                  >
                    <PieChartIcon className="h-4 w-4" />
                    Pie
                  </Button>
                  <Button 
                    variant={chartType === 'line' ? 'default' : 'outline'} 
                    size="sm"
                    onClick={() => setChartType('line')}
                    className="gap-2"
                  >
                    <TrendingUp className="h-4 w-4" />
                    Line
                  </Button>
                </div>
              </div>

              <div className="w-[200px] space-y-2">
                <label className="text-sm font-medium">Group By</label>
                <Select value={dataGrouping} onValueChange={(value) => setDataGrouping(value as DataGrouping)}>
                  <SelectTrigger>
                    <SelectValue placeholder="Select grouping" />
                  </SelectTrigger>
                  <SelectContent>
                    <SelectItem value="type">Incident Type</SelectItem>
                    <SelectItem value="town">Location/Town</SelectItem>
                    <SelectItem value="month">Month</SelectItem>
                  </SelectContent>
                </Select>
              </div>

              <div className="w-[200px] space-y-2">
                <label className="text-sm font-medium">Time Range</label>
                <Select value={timeRange} onValueChange={setTimeRange}>
                  <SelectTrigger>
                    <SelectValue placeholder="Select time range" />
                  </SelectTrigger>
                  <SelectContent>
                    <SelectItem value="all">All Time</SelectItem>
                    <SelectItem value="year">Last Year</SelectItem>
                    <SelectItem value="month">Last Month</SelectItem>
                    <SelectItem value="week">Last Week</SelectItem>
                  </SelectContent>
                </Select>
              </div>
            </div>
          </CardContent>
        </Card>

        {/* Main Chart */}
        <Card className="mb-6">
          <CardHeader>
            <CardTitle>
              {dataGrouping === 'type' && "Incidents by Type"}
              {dataGrouping === 'town' && "Incidents by Location"}
              {dataGrouping === 'month' && "Incidents by Month"}
            </CardTitle>
            <CardDescription>
              {stats?.totalIncidents} total incidents analyzed
            </CardDescription>
          </CardHeader>
          <CardContent>
            {isLoading ? (
              <div className="flex items-center justify-center h-[300px]">
                <div className="animate-spin h-8 w-8 border-4 border-primary border-t-transparent rounded-full"></div>
              </div>
            ) : chartData.length > 0 ? (
              renderChart()
            ) : (
              <div className="flex items-center justify-center h-[300px]">
                <p className="text-neutral-600">No data available</p>
              </div>
            )}
          </CardContent>
        </Card>

        {/* Metrics */}
        <div className="grid grid-cols-1 md:grid-cols-2 lg:grid-cols-3 gap-4 mb-6">
          <Card>
            <CardHeader>
              <CardTitle className="text-lg">Total Incidents</CardTitle>
            </CardHeader>
            <CardContent>
              <div className="text-3xl font-bold">{stats?.totalIncidents || 0}</div>
              <p className="text-sm text-neutral-600 mt-2">All recorded incidents</p>
            </CardContent>
          </Card>

          {metrics.map((metric, index) => (
            <Card key={index}>
              <CardHeader>
                <CardTitle className="text-lg">{metric.title}</CardTitle>
              </CardHeader>
              <CardContent>
                <div className="text-xl font-semibold">{metric.value}</div>
                <p className="text-sm text-neutral-600 mt-2">{metric.percentage} of all incidents</p>
              </CardContent>
            </Card>
          ))}
        </div>

        {/* Type Breakdown */}
        {dataGrouping === 'type' && (
          <Card>
            <CardHeader>
              <CardTitle>Incident Type Details</CardTitle>
              <CardDescription>Breakdown of each incident type</CardDescription>
            </CardHeader>
            <CardContent>
              <div className="space-y-6">
                {Object.entries(stats?.byType || {})
                  .sort((a, b) => b[1] - a[1])
                  .map(([type, count]) => {
                    const typeConfig = getIncidentTypeConfig(type);
                    const percentage = stats ? (count / stats.totalIncidents) * 100 : 0;
                    
                    return (
                      <div key={type} className="space-y-2">
                        <div className="flex items-center justify-between">
                          <div className="flex items-center">
                            <typeConfig.icon className={`h-5 w-5 ${typeConfig.color} mr-2`} />
                            <span className="font-medium">{type}</span>
                          </div>
                          <span className="text-sm text-neutral-600">{count} incidents</span>
                        </div>
                        <div className="h-2 w-full bg-neutral-100 rounded-full overflow-hidden">
                          <div 
                            className={`h-full rounded-full`}
                            style={{
                              width: `${percentage}%`,
                              backgroundColor: `var(--${typeConfig.color.replace('text-', '').replace('-500', '')})`,
                            }}
                          ></div>
                        </div>
                      </div>
                    );
                  })}
              </div>
            </CardContent>
          </Card>
        )}
      </div>
    </>
  );
}
